@@ -1,13 +1,15 @@
+from __future__ import annotations
+
 from functools import update_wrapper
 from json import JSONDecoder, JSONEncoder
-from typing import Callable, Iterable, List, Optional, TYPE_CHECKING, Union
+from typing import Callable, Iterable, List, Optional, Type, TYPE_CHECKING, Union
 
 from .static import PackageStatic
 
 if TYPE_CHECKING:
     from .app import Quart  # noqa
 
-DeferedSetupFunction = Callable[['BlueprintSetupState'], Callable]
+DeferedSetupFunction = Callable[["BlueprintSetupState"], Callable]
 
 
 class Blueprint(PackageStatic):
@@ -30,19 +32,19 @@ class Blueprint(PackageStatic):
             blueprint.
     """
 
-    json_decoder: Optional[JSONDecoder] = None
-    json_encoder: Optional[JSONEncoder] = None
+    json_decoder: Optional[Type[JSONDecoder]] = None
+    json_encoder: Optional[Type[JSONEncoder]] = None
 
     def __init__(
-            self,
-            name: str,
-            import_name: str,
-            static_folder: Optional[str]=None,
-            static_url_path: Optional[str]=None,
-            template_folder: Optional[str]=None,
-            url_prefix: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            root_path: Optional[str]=None,
+        self,
+        name: str,
+        import_name: str,
+        static_folder: Optional[str] = None,
+        static_url_path: Optional[str] = None,
+        template_folder: Optional[str] = None,
+        url_prefix: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        root_path: Optional[str] = None,
     ) -> None:
         super().__init__(import_name, template_folder, root_path, static_folder, static_url_path)
         self.name = name
@@ -51,16 +53,16 @@ class Blueprint(PackageStatic):
         self.subdomain = subdomain
 
     def route(
-            self,
-            path: str,
-            methods: Optional[List[str]]=None,
-            endpoint: Optional[str]=None,
-            defaults: Optional[dict]=None,
-            host: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            *,
-            provide_automatic_options: Optional[bool]=None,
-            strict_slashes: bool=True,
+        self,
+        path: str,
+        methods: Optional[List[str]] = None,
+        endpoint: Optional[str] = None,
+        defaults: Optional[dict] = None,
+        host: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        *,
+        provide_automatic_options: Optional[bool] = None,
+        strict_slashes: bool = True,
     ) -> Callable:
         """Add a route to the blueprint.
 
@@ -74,27 +76,36 @@ class Blueprint(PackageStatic):
             def route():
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.add_url_rule(
-                path, endpoint, func, methods, defaults=defaults, host=host, subdomain=subdomain,
-                provide_automatic_options=provide_automatic_options, strict_slashes=strict_slashes,
+                path,
+                endpoint,
+                func,
+                methods,
+                defaults=defaults,
+                host=host,
+                subdomain=subdomain,
+                provide_automatic_options=provide_automatic_options,
+                strict_slashes=strict_slashes,
             )
             return func
+
         return decorator
 
     def add_url_rule(
-            self,
-            path: str,
-            endpoint: Optional[str]=None,
-            view_func: Optional[Callable]=None,
-            methods: Optional[Iterable[str]]=None,
-            defaults: Optional[dict]=None,
-            host: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            *,
-            provide_automatic_options: Optional[bool]=None,
-            is_websocket: bool=False,
-            strict_slashes: bool=True,
+        self,
+        path: str,
+        endpoint: Optional[str] = None,
+        view_func: Optional[Callable] = None,
+        methods: Optional[Iterable[str]] = None,
+        defaults: Optional[dict] = None,
+        host: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        *,
+        provide_automatic_options: Optional[bool] = None,
+        is_websocket: bool = False,
+        strict_slashes: bool = True,
     ) -> None:
         """Add a route/url rule to the blueprint.
 
@@ -111,25 +122,32 @@ class Blueprint(PackageStatic):
             blueprint.add_url_rule('/', route)
         """
         endpoint = endpoint or view_func.__name__
-        if '.' in endpoint:
-            raise ValueError('Blueprint endpoints should not contain periods')
+        if "." in endpoint:
+            raise ValueError("Blueprint endpoints should not contain periods")
         self.record(
             lambda state: state.add_url_rule(
-                path, endpoint, view_func, methods, defaults, host, self.subdomain,
-                provide_automatic_options=provide_automatic_options, is_websocket=is_websocket,
+                path,
+                endpoint,
+                view_func,
+                methods,
+                defaults,
+                host,
+                self.subdomain,
+                provide_automatic_options=provide_automatic_options,
+                is_websocket=is_websocket,
                 strict_slashes=strict_slashes,
-            ),
+            )
         )
 
     def websocket(
-            self,
-            path: str,
-            endpoint: Optional[str]=None,
-            defaults: Optional[dict]=None,
-            host: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            *,
-            strict_slashes: bool=True,
+        self,
+        path: str,
+        endpoint: Optional[str] = None,
+        defaults: Optional[dict] = None,
+        host: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        *,
+        strict_slashes: bool = True,
     ) -> Callable:
         """Add a websocket to the blueprint.
 
@@ -143,24 +161,31 @@ class Blueprint(PackageStatic):
             async def route():
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.add_websocket(
-                path, endpoint, func, defaults=defaults, host=host, subdomain=subdomain,
+                path,
+                endpoint,
+                func,
+                defaults=defaults,
+                host=host,
+                subdomain=subdomain,
                 strict_slashes=strict_slashes,
             )
             return func
+
         return decorator
 
     def add_websocket(
-            self,
-            path: str,
-            endpoint: Optional[str]=None,
-            view_func: Optional[Callable]=None,
-            defaults: Optional[dict]=None,
-            host: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            *,
-            strict_slashes: bool=True,
+        self,
+        path: str,
+        endpoint: Optional[str] = None,
+        view_func: Optional[Callable] = None,
+        defaults: Optional[dict] = None,
+        host: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        *,
+        strict_slashes: bool = True,
     ) -> None:
         """Add a websocket rule to the blueprint.
 
@@ -177,8 +202,16 @@ class Blueprint(PackageStatic):
             blueprint.add_websocket('/', route)
         """
         return self.add_url_rule(
-            path, endpoint, view_func, {'GET'}, defaults=defaults, host=host, subdomain=subdomain,
-            provide_automatic_options=False, is_websocket=True, strict_slashes=strict_slashes,
+            path,
+            endpoint,
+            view_func,
+            {"GET"},
+            defaults=defaults,
+            host=host,
+            subdomain=subdomain,
+            provide_automatic_options=False,
+            is_websocket=True,
+            strict_slashes=strict_slashes,
         )
 
     def endpoint(self, endpoint: str) -> Callable:
@@ -194,12 +227,14 @@ class Blueprint(PackageStatic):
             def index():
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.record_once(lambda state: state.register_endpoint(endpoint, func))
             return func
+
         return decorator
 
-    def app_template_filter(self, name: Optional[str]=None) -> Callable:
+    def app_template_filter(self, name: Optional[str] = None) -> Callable:
         """Add an application wide template filter.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -212,12 +247,14 @@ class Blueprint(PackageStatic):
             def filter(value):
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.add_app_template_filter(func, name=name)
             return func
+
         return decorator
 
-    def add_app_template_filter(self, func: Callable, name: Optional[str]=None) -> None:
+    def add_app_template_filter(self, func: Callable, name: Optional[str] = None) -> None:
         """Add an application wide template filter.
 
         This is designed to be used on the blueprint directly, and
@@ -234,7 +271,7 @@ class Blueprint(PackageStatic):
         """
         self.record_once(lambda state: state.register_template_filter(func, name))
 
-    def app_template_test(self, name: Optional[str]=None) -> Callable:
+    def app_template_test(self, name: Optional[str] = None) -> Callable:
         """Add an application wide template test.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -247,12 +284,14 @@ class Blueprint(PackageStatic):
             def test(value):
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.add_app_template_test(func, name=name)
             return func
+
         return decorator
 
-    def add_app_template_test(self, func: Callable, name: Optional[str]=None) -> None:
+    def add_app_template_test(self, func: Callable, name: Optional[str] = None) -> None:
         """Add an application wide template test.
 
         This is designed to be used on the blueprint directly, and
@@ -269,7 +308,7 @@ class Blueprint(PackageStatic):
         """
         self.record_once(lambda state: state.register_template_test(func, name))
 
-    def app_template_global(self, name: Optional[str]=None) -> Callable:
+    def app_template_global(self, name: Optional[str] = None) -> Callable:
         """Add an application wide template global.
 
         This is designed to be used as a decorator, and has the same arguments
@@ -282,12 +321,14 @@ class Blueprint(PackageStatic):
             def global(value):
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.add_app_template_global(func, name=name)
             return func
+
         return decorator
 
-    def add_app_template_global(self, func: Callable, name: Optional[str]=None) -> None:
+    def add_app_template_global(self, func: Callable, name: Optional[str] = None) -> None:
         """Add an application wide template global.
 
         This is designed to be used on the blueprint directly, and
@@ -324,10 +365,9 @@ class Blueprint(PackageStatic):
     def before_websocket(self, func: Callable) -> Callable:
         """Add a before request websocket to the Blueprint.
 
-        This is designed to be used as a decorator, and has the same
-        arguments as :meth:`~quart.Quart.before_websocket`. It applies
-        only to requests that are routed to an endpoint in this
-        blueprint. An example usage,
+        This is designed to be used as a decorator, and has the same arguments
+        as :meth:`~quart.Quart.before_websocket`. It applies only to requests that
+        are routed to an endpoint in this blueprint. An example usage,
 
         .. code-block:: python
 
@@ -357,6 +397,24 @@ class Blueprint(PackageStatic):
         self.record_once(lambda state: state.app.before_request(func))
         return func
 
+    def before_app_websocket(self, func: Callable) -> Callable:
+        """Add a before request websocket to the App.
+
+        This is designed to be used as a decorator, and has the same arguments
+        as :meth:`~quart.Quart.before_websocket`. It applies to all requests to the
+        app this blueprint is registered on. An example usage,
+
+        .. code-block:: python
+
+            blueprint = Blueprint(__name__)
+            @blueprint.before_app_websocket
+            def before():
+                ...
+
+        """
+        self.record_once(lambda state: state.app.before_websocket(func))
+        return func
+
     def before_app_first_request(self, func: Callable) -> Callable:
         """Add a before request first function to the app.
 
@@ -396,10 +454,9 @@ class Blueprint(PackageStatic):
     def after_websocket(self, func: Callable) -> Callable:
         """Add an after websocket function to the Blueprint.
 
-        This is designed to be used as a decorator, and has the same
-        arguments as :meth:`~quart.Quart.after_websocket`. It applies
-        only to requests that are routed to an endpoint in this
-        blueprint. An example usage,
+        This is designed to be used as a decorator, and has the same arguments
+        as :meth:`~quart.Quart.after_websocket`. It applies only to requests that
+        are routed to an endpoint in this blueprint. An example usage,
 
         .. code-block:: python
 
@@ -428,6 +485,23 @@ class Blueprint(PackageStatic):
         self.record_once(lambda state: state.app.after_request(func))
         return func
 
+    def after_app_websocket(self, func: Callable) -> Callable:
+        """Add an after websocket function to the App.
+
+        This is designed to be used as a decorator, and has the same arguments
+        as :meth:`~quart.Quart.after_websocket`. It applies to all requests to the
+        ppe this blueprint is registerd on. An example usage,
+
+        .. code-block:: python
+
+            blueprint = Blueprint(__name__)
+            @blueprint.after_app_websocket
+            def after():
+                ...
+        """
+        self.record_once(lambda state: state.app.after_websocket(func))
+        return func
+
     def teardown_request(self, func: Callable) -> Callable:
         """Add a teardown request function to the Blueprint.
 
@@ -482,7 +556,7 @@ class Blueprint(PackageStatic):
         self.record_once(lambda state: state.app.teardown_request(func))
         return func
 
-    def errorhandler(self, error: Union[Exception, int]) -> Callable:
+    def errorhandler(self, error: Union[Type[Exception], int]) -> Callable:
         """Add an error handler function to the Blueprint.
 
         This is designed to be used as a decorator, and has the same
@@ -498,12 +572,14 @@ class Blueprint(PackageStatic):
                 ...
 
         """
+
         def decorator(func: Callable) -> Callable:
             self.register_error_handler(error, func)
             return func
+
         return decorator
 
-    def app_errorhandler(self, error: Union[Exception, int]) -> Callable:
+    def app_errorhandler(self, error: Union[Type[Exception], int]) -> Callable:
         """Add an error handler function to the App.
 
         This is designed to be used as a decorator, and has the same
@@ -517,12 +593,14 @@ class Blueprint(PackageStatic):
             def not_found():
                 ...
         """
+
         def decorator(func: Callable) -> Callable:
             self.record_once(lambda state: state.app.register_error_handler(error, func))
             return func
+
         return decorator
 
-    def register_error_handler(self, error: Union[Exception, int], func: Callable) -> None:
+    def register_error_handler(self, error: Union[Type[Exception], int], func: Callable) -> None:
         """Add an error handler function to the blueprint.
 
         This is designed to be used on the blueprint directly, and
@@ -653,37 +731,32 @@ class Blueprint(PackageStatic):
 
     def record_once(self, func: DeferedSetupFunction) -> None:
         """Used to register a deferred action that happens only once."""
-        def wrapper(state: 'BlueprintSetupState') -> None:
+
+        def wrapper(state: "BlueprintSetupState") -> None:
             if state.first_registration:
                 func(state)
+
         self.record(update_wrapper(wrapper, func))
 
     def register(
-            self,
-            app: 'Quart',
-            first_registration: bool,
-            *,
-            url_prefix: Optional[str]=None,
+        self, app: "Quart", first_registration: bool, *, url_prefix: Optional[str] = None
     ) -> None:
         """Register this blueprint on the app given."""
         state = self.make_setup_state(app, first_registration, url_prefix=url_prefix)
 
         if self.has_static_folder:
             state.add_url_rule(
-                self.static_url_path + '/<path:filename>',
-                view_func=self.send_static_file, endpoint='static',
+                self.static_url_path + "/<path:filename>",
+                view_func=self.send_static_file,
+                endpoint="static",
             )
 
         for func in self.deferred_functions:
             func(state)
 
     def make_setup_state(
-            self,
-            app: 'Quart',
-            first_registration: bool,
-            *,
-            url_prefix: Optional[str]=None,
-    ) -> 'BlueprintSetupState':
+        self, app: "Quart", first_registration: bool, *, url_prefix: Optional[str] = None
+    ) -> "BlueprintSetupState":
         """Return a blueprint setup state instance.
 
         Arguments:
@@ -707,12 +780,12 @@ class BlueprintSetupState:
     """
 
     def __init__(
-            self,
-            blueprint: Blueprint,
-            app: 'Quart',
-            first_registration: bool,
-            *,
-            url_prefix: Optional[str]=None,
+        self,
+        blueprint: Blueprint,
+        app: "Quart",
+        first_registration: bool,
+        *,
+        url_prefix: Optional[str] = None,
     ) -> None:
         self.blueprint = blueprint
         self.app = app
@@ -720,25 +793,32 @@ class BlueprintSetupState:
         self.first_registration = first_registration
 
     def add_url_rule(
-            self,
-            path: str,
-            endpoint: Optional[str]=None,
-            view_func: Optional[Callable]=None,
-            methods: Optional[Iterable[str]]=None,
-            defaults: Optional[dict]=None,
-            host: Optional[str]=None,
-            subdomain: Optional[str]=None,
-            *,
-            provide_automatic_options: Optional[bool]=None,
-            is_websocket: bool=False,
-            strict_slashes: bool=True,
+        self,
+        path: str,
+        endpoint: Optional[str] = None,
+        view_func: Optional[Callable] = None,
+        methods: Optional[Iterable[str]] = None,
+        defaults: Optional[dict] = None,
+        host: Optional[str] = None,
+        subdomain: Optional[str] = None,
+        *,
+        provide_automatic_options: Optional[bool] = None,
+        is_websocket: bool = False,
+        strict_slashes: bool = True,
     ) -> None:
         if self.url_prefix is not None:
             path = f"{self.url_prefix}{path}"
         endpoint = f"{self.blueprint.name}.{endpoint}"
         self.app.add_url_rule(
-            path, endpoint, view_func, methods, defaults, host=host, subdomain=subdomain,
-            provide_automatic_options=provide_automatic_options, is_websocket=is_websocket,
+            path,
+            endpoint,
+            view_func,
+            methods,
+            defaults,
+            host=host,
+            subdomain=subdomain,
+            provide_automatic_options=provide_automatic_options,
+            is_websocket=is_websocket,
             strict_slashes=strict_slashes,
         )
 
